import React, { useState } from 'react';
import { useNavigate } from 'react-router-dom';
import './App.css';

function App() {
  // Primary category: single selection (default “science”)
  const [primaryCategory, setPrimaryCategory] = useState('science');
  // Additional categories: multiple selection (initially empty)
  const [additionalCategories, setAdditionalCategories] = useState([]);
  // Controls whether the primary dropdown is open
  const [primaryDropdownOpen, setPrimaryDropdownOpen] = useState(false);
  // The search query state
  const [query, setQuery] = useState('');

  const navigate = useNavigate();

  // Existing maps for images, header, backgrounds, etc.
  const imageMap = {
    science: '/main.png',
    philosophy: '/philosophy.png',
    tech: '/tech.png',
    discover: '/web.png',
    multi: '/multi.png'
  };

  const headerMap = {
    science: '/header.png',
    philosophy: '/header_philosophy.png',
    tech: '/header_tech.png',
    discover: '/header_web.png',
    multi: '/header_multi.png'
  };

  // Use #ffffff for multi so it remains white
  const appBgMap = {
    science: '#fcfcff',
    philosophy: '#ffffff',
    tech: '#ffffff',
    discover: '#ffffff',
    multi: '#ffffff'
  };

  const headerBgMap = {
    science: '#f8f8f8',
    philosophy: '#ffffff',
    tech: '#ffffff',
    discover: '#ffffff',
    multi: '#ffffff'
  };

  // Mapping keys to display names
  const displayNames = {
    science: "The Universe of Science",
    philosophy: "Philosophy and Logic",
    tech: "Computers and Technology",
    discover: "Discover the Web"
  };

  // Define the complete list of available category keys.
  const allCategories = ["science", "philosophy", "tech", "discover"];

  // For the header, use "multi" if any additional category is selected; else use primary.
  const currentCategory =
    additionalCategories.length > 0 ? "multi" : primaryCategory;

  // Filter out the primary category from the additional dropdown’s options
  const additionalOptions = allCategories.filter(cat => cat !== primaryCategory);

  // Sort additional options so that selected ones (in the order selected) appear at top
  additionalOptions.sort((a, b) => {
    const aSelected = additionalCategories.includes(a);
    const bSelected = additionalCategories.includes(b);
    if (aSelected && !bSelected) return -1;
    if (!aSelected && bSelected) return 1;
    if (aSelected && bSelected) {
      return additionalCategories.indexOf(a) - additionalCategories.indexOf(b);
    }
    return a.localeCompare(b);
  });

  // On form submit, build comma‑separated list: primaryCategory + additionalCategories
  const handleSubmit = (e) => {
    e.preventDefault();
    const catParam =
      primaryCategory +
      (additionalCategories.length > 0 ? ',' + additionalCategories.join(',') : '');
    navigate(`/results?q=${encodeURIComponent(query)}&cat=${catParam}`);
  };

  // Single‑select for the primary bar
  const handlePrimaryCategorySelect = (cat) => {
    setPrimaryCategory(cat);
    // If the chosen cat was in the additional list, remove it
    setAdditionalCategories(prev => prev.filter(c => c !== cat));
    setPrimaryDropdownOpen(false);
  };

  // Toggle selection for the additional bar
  const handleAdditionalToggle = (cat) => {
    setAdditionalCategories(prev => {
      if (prev.includes(cat)) {
        return prev.filter(c => c !== cat);
      } else {
        return [...prev, cat];
      }
    });
  };

  return (
    <div className="App" style={{ backgroundColor: appBgMap[currentCategory] }}>
      <header className="header-bar" style={{ backgroundColor: headerBgMap[currentCategory] }}>
        <img src={headerMap[currentCategory]} alt="Header" />
      </header>
      <main className="main-content">
        <img id="mascot-img" src={imageMap[currentCategory]} alt="Mascot" />
        <div id="tagline">a search engine for the rest of us</div>

        <form className="search-form" onSubmit={handleSubmit}>
          <div className="search-container">
            <input
              type="text"
              name="search"
              placeholder="Search..."
              value={query}
              onChange={(e) => setQuery(e.target.value)}
            />
          </div>

          {/* PRIMARY CATEGORY DROPDOWN (single select) */}
          <div
            className={`custom-dropdown primary-dropdown ${primaryDropdownOpen ? 'open' : ''}`}
            onClick={() => setPrimaryDropdownOpen(!primaryDropdownOpen)}
          >
            <div className="dropdown-selected">
              {displayNames[primaryCategory]}
            </div>
            {primaryDropdownOpen && (
              <div className="dropdown-items">
                {allCategories.map(cat => (
                  <div
                    key={cat}
                    className="dropdown-item"
                    onClick={(e) => {
                      e.stopPropagation();
                      handlePrimaryCategorySelect(cat);
                    }}
                  >
                    {displayNames[cat]}
                  </div>
                ))}
              </div>
            )}
          </div>

          {/* ADDITIONAL CATEGORIES DROPDOWN (multiple select; remains open) */}
          <div className="custom-dropdown additional-dropdown">
            <div className="dropdown-selected">
              {/* Always show "Select Additional Categories" even if some are chosen */}
              Select Additional Categories
            </div>
            <div className="dropdown-items">
              {additionalOptions.map(cat => {
                const isSelected = additionalCategories.includes(cat);
                return (
                  <div
                    key={cat}
                    className={`dropdown-item ${isSelected ? 'dropdown-item-selected' : ''}`}
                    onClick={(e) => {
                      e.stopPropagation();
                      handleAdditionalToggle(cat);
                    }}
                  >
                    {displayNames[cat]}
                  </div>
                );
              })}
            </div>
          </div>
        </form>
      </main>
      <footer className="footer">
        <a href="https://git.tsugutsugugo.com/explore/repos">Source</a> |{' '}
        <a href="https://docs.tsugutsugugo.com/">Docs</a>
      </footer>
    </div>
  );
}

export default App;
