// MultiCategoryResults.jsx
import React, { useEffect, useState } from "react";
import { Link } from "react-router-dom";
import "./App.css";

function MultiCategoryResults({ query, categories }) {
  const [results, setResults] = useState([]);
  const [loading, setLoading] = useState(false);
  const [videoResults, setVideoResults] = useState([]);
  const [videoLoading, setVideoLoading] = useState(false);
  const [currentPage, setCurrentPage] = useState(1);
  const [viewMode, setViewMode] = useState("web");
  const [visitedLinks, setVisitedLinks] = useState(() => {
    const stored = localStorage.getItem("visitedLinks");
    return stored ? JSON.parse(stored) : [];
  });
  const resultsPerPage = 10;
  const numCats = categories.length;

  // Define pages per category according to the rules:
  // 1 category: 9 pages, 2 categories: 4 pages each, 3 categories: 3 pages each, 4+ categories: 2 pages each.
  let pagesPerCat;
  if (numCats === 1) {
    pagesPerCat = 9;
  } else if (numCats === 2) {
    pagesPerCat = 4;
  } else if (numCats === 3) {
    pagesPerCat = 3;
  } else {
    pagesPerCat = 2;
  }
  const totalPages = numCats > 1 ? numCats * pagesPerCat : pagesPerCat;

  // Determine which category to fetch.
  // For multiple categories, we rotate through them.
  const categoryIndex = (currentPage - 1) % numCats;
  const pageForCategory = Math.floor((currentPage - 1) / numCats) + 1;
  const fetchCategory = categories[categoryIndex];

  // For header and styling, always use the primary category (the first element).
  const primaryCategory = categories[0];
  const headerCategory = primaryCategory;

  // URL maps
  const sourceMap = {
    science: "https://science.tsugutsugugo.com/yacysearch.json",
    philosophy: "https://sophos.tsugutsugugo.com/yacysearch.json",
    tech: "https://tech.tsugutsugugo.com/yacysearch.json",
    discover: "https://web.tsugutsugugo.com/yacysearch.json",
  };

  const headerMap = {
    science: "/header.png",
    philosophy: "/header_philosophy.png",
    tech: "/header_tech.png",
    discover: "/header_web.png",
    multi: "/header_multi.png",
  };

  const headerBgMap = {
    science: "#f8f8f8",
    philosophy: "#ffffff",
    tech: "#ffffff",
    discover: "#ffffff",
    multi: "#ffffff",
  };

  const appBgMap = {
    science: "#fcfcff",
    philosophy: "#ffffff",
    tech: "#ffffff",
    discover: "#ffffff",
    multi: "#ffffff",
  };

  // Determine if mobile based on window width.
  const isMobile = window.innerWidth < 600;

  // Fetch results based on current page.
  useEffect(() => {
    const fetchResults = async () => {
      setLoading(true);
      // Calculate the starting record number for this page.
      const startRecord = (pageForCategory - 1) * resultsPerPage;
      // Build the fetch URL.
      const url = `${sourceMap[fetchCategory]}?query=${encodeURIComponent(
        query
      )}&resource=global&urlmaskfilter=.*&prefermaskfilter=&nav=all&maximumRecords=${resultsPerPage}&startRecord=${startRecord}`;

      try {
        const response = await fetch(url);
        if (response.ok) {
          const data = await response.json();
          const items = data.channels[0]?.items || [];
          if (isMobile) {
            // For mobile: on page 1, set the results; on subsequent pages, append.
            if (currentPage === 1) {
              setResults(items);
            } else {
              setResults((prev) => [...prev, ...items]);
            }
          } else {
            // Desktop behavior:
            if (numCats > 1) {
              // Multi‑category desktop fetch simply replaces results.
              setResults(items);
            } else {
              // For desktop single category, aggregate results over all pages.
              let allResults = [];
              for (let page = 0; page < pagesPerCat; page++) {
                const record = page * resultsPerPage;
                const responsePage = await fetch(
                  `${sourceMap[primaryCategory]}?query=${encodeURIComponent(
                    query
                  )}&resource=global&urlmaskfilter=.*&prefermaskfilter=&nav=all&maximumRecords=${resultsPerPage}&startRecord=${record}`
                );
                if (!responsePage.ok) continue;
                const dataPage = await responsePage.json();
                allResults = [
                  ...allResults,
                  ...(dataPage.channels[0]?.items || []),
                ];
              }
              setResults(allResults);
            }
          }
        } else {
          console.error("Error response from server");
        }
      } catch (error) {
        console.error("Error fetching multi-category results:", error);
      }
      setLoading(false);
    };

    if (viewMode === "web") {
      fetchResults();
    }
  }, [
    query,
    currentPage,
    viewMode,
    numCats,
    pageForCategory,
    fetchCategory,
    primaryCategory,
    resultsPerPage,
    pagesPerCat,
    isMobile,
  ]);

  // Compute the maximum number of pages.
  // For mobile, we want to stop once we have reached the page limit.
  const computedPages = isMobile
    ? numCats > 1
      ? totalPages
      : pagesPerCat
    : numCats > 1
    ? totalPages
    : Math.ceil(results.length / resultsPerPage);

  // For mobile devices, add a scroll event listener to load more results when nearing the bottom.
  useEffect(() => {
    if (isMobile && viewMode === "web") {
      const handleScroll = () => {
        const scrollTop =
          document.documentElement.scrollTop || document.body.scrollTop;
        const windowHeight = window.innerHeight;
        const fullHeight = document.documentElement.offsetHeight;
        // When near the bottom and not already loading, request the next page if available.
        if (scrollTop + windowHeight >= fullHeight - 50) {
          if (!loading && currentPage < computedPages) {
            setCurrentPage((prevPage) => prevPage + 1);
          }
        }
      };

      window.addEventListener("scroll", handleScroll);
      return () => {
        window.removeEventListener("scroll", handleScroll);
      };
    }
  }, [isMobile, viewMode, loading, currentPage, computedPages]);

  // For desktop pagination (when not mobile).
  const paginate = (page) => setCurrentPage(page);

  const fetchVideoResults = async () => {
    setVideoLoading(true);
    try {
      const response = await fetch(
        `https://stream.tsugutsugugo.com/api/v1/videos?search=${encodeURIComponent(
          query
        )}`
      );
      if (!response.ok) throw new Error("Video API error");
      const data = await response.json();
      setVideoResults(data.data || []);
    } catch (error) {
      console.error("Error fetching video results:", error);
    } finally {
      setVideoLoading(false);
    }
  };

  const handleLinkClick = (url) => {
    setVisitedLinks((prev) => {
      if (!prev.includes(url)) {
        const newVisited = [...prev, url];
        localStorage.setItem("visitedLinks", JSON.stringify(newVisited));
        return newVisited;
      }
      return prev;
    });
  };

  return (
    <div
      className="App results-page"
      style={{ backgroundColor: appBgMap[headerCategory] }}
    >
      <header
        className="header-bar"
        style={{ backgroundColor: headerBgMap[headerCategory] }}
      >
        <Link to="/">
          <img
            src={headerMap[headerCategory]}
            alt={`${headerCategory} Header`}
          />
        </Link>
      </header>
      <main className="main-content">
        <div id="tagline">
          Search Results for <strong>{query}</strong>,{" "}
          {viewMode === "web" ? (
            <span
              className="link-switch"
              onClick={async () => {
                await fetchVideoResults();
                setViewMode("video");
              }}
            >
              browse videos
            </span>
          ) : (
            <span className="link-switch" onClick={() => setViewMode("web")}>
              browse web results
            </span>
          )}
        </div>
        {viewMode === "web" ? (
          loading && currentPage === 1 ? (
            <p>Loading results...</p>
          ) : results.length > 0 ? (
            <>
              <ul className="results-list">
                {results.map((item, idx) => (
                  <li key={idx} className="result-item">
                    <a
                      href={item.link}
                      target="_blank"
                      rel="noopener noreferrer"
                      onClick={() => handleLinkClick(item.link)}
                      className={`result-title ${
                        visitedLinks.includes(item.link) ? "visited-link" : ""
                      }`}
                    >
                      {item.title}
                    </a>
                    <p dangerouslySetInnerHTML={{ __html: item.description }} />
                    <a
                      href={`https://web.archive.org/web/*/${item.link}`}
                      target="_blank"
                      rel="noopener noreferrer"
                      className="cache-link"
                    >
                      Cache
                    </a>
                  </li>
                ))}
              </ul>
              {/* Desktop: Render pagination buttons */}
              {!isMobile &&
                window.innerWidth >= 600 &&
                computedPages > 1 && (
                  <div className="pagination">
                    {[...Array(computedPages)].map((_, idx) => (
                      <button
                        key={idx}
                        onClick={() => paginate(idx + 1)}
                        className={`page-btn ${
                          currentPage === idx + 1 ? "active-page" : ""
                        }`}
                      >
                        {idx + 1}
                      </button>
                    ))}
                  </div>
                )}
              {/* Mobile: Optionally indicate that more results are loading */}
              {isMobile && loading && currentPage > 1 && (
                <p>Loading more results...</p>
              )}
            </>
          ) : (
            <p>No search results found.</p>
          )
        ) : videoLoading ? (
          <p>Loading videos...</p>
        ) : videoResults && videoResults.length > 0 ? (
          <ul className="results-list">
            {videoResults.map((video, idx) => (
              <li key={idx} className="result-item">
                <a
                  href={video.url}
                  target="_blank"
                  rel="noopener noreferrer"
                  className="result-title"
                >
                  {video.name}
                </a>
                <p dangerouslySetInnerHTML={{ __html: video.description }} />
              </li>
            ))}
          </ul>
        ) : (
          <p>No video results found.</p>
        )}
      </main>
      <footer className="footer">
        <a href="https://git.tsugutsugugo.com/explore/repos">Source</a> |{" "}
        <a href="https://docs.tsugutsugugo.com/">Docs</a>
      </footer>
    </div>
  );
}

export default MultiCategoryResults;
