// SingleCategoryResults.jsx
import React, { useEffect, useState } from 'react';
import { Link } from 'react-router-dom';
import './App.css';

function SingleCategoryResults({ query, primaryCategory }) {
  const [results, setResults] = useState([]);
  const [loading, setLoading] = useState(true);
  const [videoResults, setVideoResults] = useState([]);
  const [videoLoading, setVideoLoading] = useState(false);
  const [currentPage, setCurrentPage] = useState(1);
  const [viewMode, setViewMode] = useState('web'); // "web" or "video"
  const [visitedLinks, setVisitedLinks] = useState(() => {
    const stored = localStorage.getItem('visitedLinks');
    return stored ? JSON.parse(stored) : [];
  });
  const resultsPerPage = 10;


  const sourceMap = {
    science: 'https://science.tsugutsugugo.com/yacysearch.json',
    philosophy: 'https://sophos.tsugutsugugo.com/yacysearch.json',
    tech: 'https://tech.tsugutsugugo.com/yacysearch.json',
    discover: 'https://web.tsugutsugugo.com/yacysearch.json',
  };

  const headerMap = {
    science: '/header.png',
    philosophy: '/header_philosophy.png',
    tech: '/header_tech.png',
    discover: '/header_web.png',
  };

  const headerBgMap = {
    science: '#f8f8f8',
    philosophy: '#ffffff',
    tech: '#ffffff',
    discover: '#ffffff',
  };

  const appBgMap = {
    science: '#fcfcff',
    philosophy: '#ffffff',
    tech: '#ffffff',
    discover: '#ffffff',
  };

  // For header, always use primaryCategory.
  const headerCategory = primaryCategory;

  // Fetch aggregated results over 10 pages.
  useEffect(() => {
    const fetchSingleResults = async () => {
      setLoading(true);
      let allResults = [];
      for (let page = 0; page < 10; page++) {
        const startRecord = page * resultsPerPage;
        try {
          const response = await fetch(
            `${sourceMap[primaryCategory]}?query=${encodeURIComponent(query)}&resource=global&urlmaskfilter=.*&prefermaskfilter=&nav=all&maximumRecords=${resultsPerPage}&startRecord=${startRecord}`
          );
          if (response.ok) {
            const data = await response.json();
            allResults = [...allResults, ...(data.channels[0]?.items || [])];
          }
        } catch (error) {
          console.error("Error fetching single-category results:", error);
        }
      }
      setResults(allResults);
      setLoading(false);
    };

    if (viewMode === "web") {
      fetchSingleResults();
    }
  }, [query, primaryCategory, viewMode]);

  // Video results logic (unchanged)
  const fetchVideoResults = async () => {
    setVideoLoading(true);
    try {
      const response = await fetch(`https://stream.tsugutsugugo.com/api/v1/videos?search=${encodeURIComponent(query)}`);
      if (!response.ok) throw new Error("Video API error");
      const data = await response.json();
      setVideoResults(data.data || []);
    } catch (error) {
      console.error("Error fetching video results:", error);
    } finally {
      setVideoLoading(false);
    }
  };

  const computedPages = Math.ceil(results.length / resultsPerPage);
  const paginate = (page) => setCurrentPage(page);

  const handleLinkClick = (url) => {
    setVisitedLinks(prev => {
      if (!prev.includes(url)) {
        const newVisited = [...prev, url];
        localStorage.setItem("visitedLinks", JSON.stringify(newVisited));
        return newVisited;
      }
      return prev;
    });
  };

  return (
    <div className="App results-page" style={{ backgroundColor: appBgMap[headerCategory] }}>
      <header className="header-bar" style={{ backgroundColor: headerBgMap[headerCategory] }}>
        <Link to="/">
          <img src={headerMap[headerCategory]} alt={`${headerCategory} Header`} />
        </Link>
      </header>
      <main className="main-content">
        <div id="tagline">
          Search Results for <strong>{query}</strong>,{" "}
          {viewMode === "web" ? (
            <span
              className="link-switch"
              onClick={async () => { await fetchVideoResults(); setViewMode("video"); }}
            >
              browse videos
            </span>
          ) : (
            <span className="link-switch" onClick={() => setViewMode("web")}>
              browse web results
            </span>
          )}
        </div>
        {viewMode === "web" ? (
          loading ? (
            <p>Loading results...</p>
          ) : results.length > 0 ? (
            <>
              <ul className="results-list">
                {results
                  .slice(
                    window.innerWidth < 600 ? 0 : (currentPage - 1) * resultsPerPage,
                    window.innerWidth < 600 ? results.length : currentPage * resultsPerPage
                  )
                  .map((item, idx) => (
                    <li key={idx} className="result-item">
                      <a
                        href={item.link}
                        target="_blank"
                        rel="noopener noreferrer"
                        onClick={() => handleLinkClick(item.link)}
                        className={`result-title ${visitedLinks.includes(item.link) ? "visited-link" : ""}`}
                      >
                        {item.title}
                      </a>
                      <p dangerouslySetInnerHTML={{ __html: item.description }} />
                      <a
                        href={`https://web.archive.org/web/*/${item.link}`}
                        target="_blank"
                        rel="noopener noreferrer"
                        className="cache-link"
                      >
                        Cache
                      </a>
                    </li>
                  ))}
              </ul>
              {window.innerWidth >= 600 && computedPages > 1 && (
                <div className="pagination">
                  {[...Array(computedPages)].map((_, idx) => (
                    <button
                      key={idx}
                      onClick={() => paginate(idx + 1)}
                      className={`page-btn ${currentPage === idx + 1 ? "active-page" : ""}`}
                    >
                      {idx + 1}
                    </button>
                  ))}
                </div>
              )}
            </>
          ) : (
            <p>No search results found.</p>
          )
        ) : videoLoading ? (
          <p>Loading videos...</p>
        ) : videoResults && videoResults.length > 0 ? (
          <ul className="results-list">
            {videoResults.map((video, idx) => (
              <li key={idx} className="result-item">
                <a
                  href={video.url}
                  target="_blank"
                  rel="noopener noreferrer"
                  className="result-title"
                >
                  {video.name}
                </a>
                <p dangerouslySetInnerHTML={{ __html: video.description }} />
              </li>
            ))}
          </ul>
        ) : (
          <p>No video results found.</p>
        )}
      </main>
      <footer className="footer">
        <a href="https://git.tsugutsugugo.com/explore/repos">Source</a> |{" "}
        <a href="https://docs.tsugutsugugo.com/">Docs</a>
      </footer>
    </div>
  );
}

export default SingleCategoryResults;
