import React, { useEffect, useState } from 'react';

function YacySearchResults({ category, query }) {
  const [results, setResults] = useState([]);
  const [loading, setLoading] = useState(false);
  const [page, setPage] = useState(0); // zero-indexed page number
  const resultsPerPage = 50; // number of results per page

  // Map categories to their corresponding API endpoint
  const endpoints = {
    science:    "https://science.tsugutsugugo.com/yacysearch.json?query=",
    philosophy: "https://sophos.tsugutsugugo.com/yacysearch.json?query=",
    tech:       "https://tech.tsugutsugugo.com/yacysearch.json?query=",
    discover:   "https://web.tsugutsugugo.com/yacysearch.json?query="
  };

  useEffect(() => {
    if (!query) {
      setResults([]);
      return;
    }
    setLoading(true);
    const startRecord = page * resultsPerPage;
    const baseUrl = endpoints[category] || endpoints.science;
    const apiUrl = `${baseUrl}${encodeURIComponent(query)}&resource=global&urlmaskfilter=.*&prefermaskfilter=&nav=all&startRecord=${startRecord}&maximumRecords=${resultsPerPage}`;

    fetch(apiUrl)
      .then(res => res.json())
      .then(data => {
        const items = data.channel?.item || [];
        const parsedResults = items.map((item) => ({
          title:       item.title,
          link:        item.link,
          description: item.description,
          pubDate:     item.pubDate
        }));
        setResults(parsedResults);
      })
      .catch(err => {
        console.error("Error fetching search results:", err);
        setResults([]);
      })
      .finally(() => {
        setLoading(false);
      });
  }, [category, query, page]);

  const handleNext = () => {
    setPage(page + 1);
  };

  const handlePrev = () => {
    if (page > 0) setPage(page - 1);
  };

  return (
    <div className="search-results">
      {loading && <p>Loading results...</p>}
      {!loading && results.length === 0 && query && (
        <p>No results found for "<strong>{query}</strong>".</p>
      )}
      {results.map(result => (
        <div key={result.link} className="result-item">
          <h3 className="result-title">
            <a href={result.link} target="_blank" rel="noopener noreferrer">
              {result.title}
            </a>
          </h3>
          <p
            className="result-description"
            dangerouslySetInnerHTML={{ __html: result.description }}
          />
          <p className="result-meta">
            <span className="result-date">{result.pubDate}</span>{" "}
            |{" "}
            <a
              href={`https://web.archive.org/web/*/${encodeURIComponent(result.link)}`}
              target="_blank"
              rel="noopener noreferrer"
            >
              Cache
            </a>
          </p>
        </div>
      ))}
      <div className="pagination">
        {page > 0 && <button onClick={handlePrev}>Previous</button>}
        {results.length === resultsPerPage && <button onClick={handleNext}>Next</button>}
      </div>
    </div>
  );
}

export default YacySearchResults;
